@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999-2003 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 1999-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
@@ -193,7 +193,7 @@ static int	SetVolumeUUIDRaw(const char *deviceNamePtr, VolumeUUID *volumeUUIDPtr
 static int	SetVolumeUUIDAttr(const char *path, VolumeUUID *volumeUUIDPtr);
 static int	SetVolumeUUID(const char *deviceNamePtr, VolumeUUID *volumeUUIDPtr);
 static int	GetEmbeddedHFSPlusVol(HFSMasterDirectoryBlock * hfsMasterDirectoryBlockPtr, off_t * startOffsetPtr);
-static int	GetNameFromHFSPlusVolumeStartingAt(int fd, off_t hfsPlusVolumeOffset, char * name_o);
+static int	GetNameFromHFSPlusVolumeStartingAt(int fd, off_t hfsPlusVolumeOffset, unsigned char * name_o);
 static int	GetBTreeNodeInfo(int fd, off_t hfsPlusVolumeOffset, u_int32_t blockSize,
 							u_int32_t extentCount, const HFSPlusExtentDescriptor *extentList,
 							u_int32_t *nodeSize, u_int32_t *firstLeafNode);
@@ -683,7 +683,7 @@ DoProbe(char *deviceNamePtr)
 
 		/* Some poorly mastered HFS CDs have an empty MDB name field! */
 		if (mdbPtr->drVN[0] == '\0') {
-			strcpy(&mdbPtr->drVN[1], gHFS_FS_NAME_NAME);
+			strcpy((char *)&mdbPtr->drVN[1], gHFS_FS_NAME_NAME);
 			mdbPtr->drVN[0] = strlen(gHFS_FS_NAME_NAME);
 		}
 
@@ -744,7 +744,7 @@ DoProbe(char *deviceNamePtr)
 	}
 
 	if (FSUR_IO_SUCCESS == result) {
-		char *s;
+		unsigned char *s;
 		
 		/* Change slashes to colons in the volume name */
 		for (s=volnameUTF8; *s; ++s) {
@@ -753,7 +753,7 @@ DoProbe(char *deviceNamePtr)
 		}
 
 		/* Print the volume name to standard output */
-		write(1, volnameUTF8, strlen(volnameUTF8));
+		write(1, volnameUTF8, strlen((char *)volnameUTF8));
 		result = FSUR_RECOGNIZED;
 	}
 
@@ -1631,7 +1631,7 @@ Return:
  */
 
 static int
-GetNameFromHFSPlusVolumeStartingAt(int fd, off_t hfsPlusVolumeOffset, char * name_o)
+GetNameFromHFSPlusVolumeStartingAt(int fd, off_t hfsPlusVolumeOffset, unsigned char * name_o)
 {
     int					result = FSUR_IO_SUCCESS;
     u_int32_t				blockSize;
@@ -1768,7 +1768,7 @@ GetNameFromHFSPlusVolumeStartingAt(int fd, off_t hfsPlusVolumeOffset, char * nam
 	    }
 	    swapped->unicode[i] = 0;
 	    cfstr = CFStringCreateWithCharacters(kCFAllocatorDefault, swapped->unicode, swapped->length);
-	    (void) CFStringGetCString(cfstr, name_o, NAME_MAX, kCFStringEncodingUTF8);
+	    (void) CFStringGetCString(cfstr, (char *)name_o, NAME_MAX, kCFStringEncodingUTF8);
 	    CFRelease(cfstr);
 	    free(swapped);
 	}
@@ -1866,6 +1866,7 @@ GetCatalogOverflowExtents(int fd, off_t hfsPlusVolumeOffset,
 	off_t offset;
 	u_int32_t nodeSize;
 	u_int32_t leafNode;
+	u_int32_t blockSize;
     BTNodeDescriptor * bTreeNodeDescriptorPtr;
 	HFSPlusExtentDescriptor * extents;
 	size_t listsize;
@@ -1873,14 +1874,15 @@ GetCatalogOverflowExtents(int fd, off_t hfsPlusVolumeOffset,
 	int i;
 	int result;
 
+	blockSize = NXSwapBigLongToHost(volHdrPtr->blockSize);
 	listsize = *catalogExtCount * sizeof(HFSPlusExtentDescriptor);
 	extents = *catalogExtents;
 	offset = (off_t)volHdrPtr->extentsFile.extents[0].startBlock *
-		    (off_t)volHdrPtr->blockSize;
+		    (off_t)blockSize;
 
 	/* Read the header node of the extents B-Tree */
 
-	result = GetBTreeNodeInfo(fd, hfsPlusVolumeOffset, volHdrPtr->blockSize,
+	result = GetBTreeNodeInfo(fd, hfsPlusVolumeOffset, blockSize,
 			kHFSPlusExtentDensity, volHdrPtr->extentsFile.extents,
 		    &nodeSize, &leafNode);
 	if (result != FSUR_IO_SUCCESS || leafNode == 0)
@@ -1902,7 +1904,7 @@ GetCatalogOverflowExtents(int fd, off_t hfsPlusVolumeOffset,
 
 again:
 	result = ReadFile(fd, bufPtr, offset, nodeSize,
-					hfsPlusVolumeOffset, volHdrPtr->blockSize,
+					hfsPlusVolumeOffset, blockSize,
 					kHFSPlusExtentDensity, volHdrPtr->extentsFile.extents);
 	if ( result == FSUR_IO_FAIL ) {
 #if TRACE_HFS_UTIL
@@ -1939,7 +1941,7 @@ again:
 		/* grow list and copy additional extents */
 		listsize += sizeof(HFSPlusExtentRecord);
 		extents = (HFSPlusExtentDescriptor *) realloc(extents, listsize);
-		bcopy(p + k->keyLength + sizeof(u_int16_t),
+		bcopy(p + NXSwapBigShortToHost(k->keyLength) + sizeof(u_int16_t),
 			&extents[*catalogExtCount], sizeof(HFSPlusExtentRecord));
 
 		*catalogExtCount += kHFSPlusExtentDensity;
@@ -2326,7 +2328,7 @@ void GenerateVolumeUUID(VolumeUUID *newVolumeID) {
 	int sysdata;
 	char sysctlstring[128];
 	size_t datalen;
-	struct loadavg sysloadavg;
+	double sysloadavg[3];
 	struct vmtotal sysvmtotal;
 	
 	do {
@@ -2375,10 +2377,8 @@ void GenerateVolumeUUID(VolumeUUID *newVolumeID) {
 		SHA1_Update(&context, sysctlstring, datalen);
 
 		/* The system's load average: */
-		mib[0] = CTL_VM;
-		mib[1] = VM_LOADAVG;
 		datalen = sizeof(sysloadavg);
-		sysctl(mib, 2, &sysloadavg, &datalen, NULL, 0);
+		getloadavg(sysloadavg, 3);
 		SHA1_Update(&context, &sysloadavg, datalen);
 
 		/* The system's VM statistics: */
